[PATCH] D92599: Fix for Bug 48055.

Hiroshi Yamauchi via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Fri Dec 4 10:29:48 PST 2020


yamauchi updated this revision to Diff 309576.
yamauchi marked an inline comment as done.
yamauchi added a comment.

Address comment.


Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D92599/new/

https://reviews.llvm.org/D92599

Files:
  llvm/lib/Transforms/Utils/Evaluator.cpp
  llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll


Index: llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll
===================================================================
--- /dev/null
+++ llvm/test/Transforms/GlobalOpt/evaluate-bitcast-4.ll
@@ -0,0 +1,29 @@
+; PR48055. Check that this does not crash.
+; RUN: opt -globalopt %s -disable-output
+target datalayout = "e-m:e-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:128-n8:16:32:64-S128"
+target triple = "x86_64-unknown-linux-gnu"
+
+%struct.g = type opaque
+%struct.a = type { i32 (...)** }
+
+ at l = dso_local global i32 0, align 4
+ at h = external dso_local global %struct.g, align 1
+ at llvm.global_ctors = appending global [1 x { i32, void ()*, i8* }] [{ i32, void ()*, i8* } { i32 65535, void ()* @_GLOBAL__sub_I_bug48055.cc, i8* null }]
+
+; Function Attrs: uwtable
+define internal void @__cxx_global_var_init() {
+entry:
+  %vtable = load i32 (%struct.a*)**, i32 (%struct.a*)*** bitcast (%struct.g* @h to i32 (%struct.a*)***), align 1
+  %0 = load i32 (%struct.a*)*, i32 (%struct.a*)** %vtable, align 8
+  %call = call i32 %0(%struct.a* nonnull dereferenceable(8) bitcast (%struct.g* @h to %struct.a*))
+  store i32 %call, i32* @l, align 4
+  ret void
+}
+
+; Function Attrs: uwtable
+define internal void @_GLOBAL__sub_I_bug48055.cc() {
+entry:
+  call void @__cxx_global_var_init()
+  ret void
+}
+
Index: llvm/lib/Transforms/Utils/Evaluator.cpp
===================================================================
--- llvm/lib/Transforms/Utils/Evaluator.cpp
+++ llvm/lib/Transforms/Utils/Evaluator.cpp
@@ -183,11 +183,11 @@
                        std::function<Constant *(Constant *)> Func) {
   Constant *Val;
   while (!(Val = Func(Ptr))) {
     // If Ty is a struct, we can convert the pointer to the struct
     // into a pointer to its first member.
     // FIXME: This could be extended to support arrays as well.
     Type *Ty = cast<PointerType>(Ptr->getType())->getElementType();
-    if (!isa<StructType>(Ty))
+    if (!isa<StructType>(Ty) || cast<StructType>(Ty)->isOpaque())
       break;
 
     IntegerType *IdxTy = IntegerType::get(Ty->getContext(), 32);


-------------- next part --------------
@@ -183,11 +183,11 @@
   Constant *Val;
   while (!(Val = Func(Ptr))) {
-    // If Ty is a struct, we can convert the pointer to the struct
+    // If Ty is a non-opaque struct, we can convert the pointer to the struct
     // into a pointer to its first member.
     // FIXME: This could be extended to support arrays as well.
A non-text attachment was scrubbed...
Name: D92599.309576.patch
Type: text/x-patch
Size: 2163 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20201204/02fe48be/attachment.bin>


More information about the llvm-commits mailing list